import java.io.*;

public class Reliable {
    private Link link;
    private int timeout;
    
    /**
     * Constructor for the class Reliable
     *
     * @param l The Link object to be used for sending and receiving packets
     *        windowsize The size of the sliding window
     *        timeout The time to wait for an ACK before timing out (in milliseconds)
     */
    public Reliable(Link l, int windowsize, int timeout) {
	link=l;
	this.timeout = timeout;
    }

    /**
     * Transfers a file to the server.
     *
     * @param fname The name of the file to transfer.
     */
    public void transfer(String fname, int startseq) {
	DataGen dg = new DataGen(fname);
	Packet p, ack;


	/*
	 * When you run against the stop and wait 
	 * server the initial seq should be 0
	 *
	 * When running against the slidingwindow 
	 * server the initial seqno should be startseq
	 * int seqno=startseq;
	 */ 

	int seqno=0;
	boolean success = false;
	
	byte data[]=null;

	while((data = dg.generateData()) != null) {
	    p = new Packet(seqno, data.length, false, data);
	    System.out.println("Sending packet "+p);
	    while(!success) {
		link.sendPacket(p);
		try {
		    ack = link.getPacket(timeout);
		    if(ack.seqno() == seqno+1) {
			success = true;
		    }
		} catch (InterruptedIOException e) {
		    System.out.println("Packet "+seqno+" timed out. Retransmitting.");
		}
	    }
	    seqno++;
	    success = false;
	}

	// Done!!!
        // You should not need to change anything after this point


	while (!success) {
	    link.sendPacket(new Packet(-1, 0, false, null));
	    try {
		ack = link.getPacket(timeout); // Set timeout differently
		if (ack.seqno() == -1) { // #### BV: Check that it's the correct ack
		    System.out.println((ack.ack()?"Success":"Failure")+": "+ack.getData()[0]+"% of the maximum amount of points.");
		    success = true;
		} else
		    System.out.println("Ignoring ack "+ ack);
	    } catch (InterruptedIOException e) {
		System.out.println("Final ack timeout, retrying");
	    }
	}
    }
}
























